/* KaffeEncoder.java -- Encodes various encodings using kaffe's encoder
   Copyright (C) 1998 Free Software Foundation, Inc.

This file is part of GNU Classpath.

GNU Classpath is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.
 
GNU Classpath is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Classpath; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version. */


package gnu.java.io.encode;

import java.io.OutputStream;
import java.io.CharConversionException;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import kaffe.io.CharToByteConverter;

/**
  *
  * @version 0.0
  *
  * @author Ito Kazumitsu (kaz@maczuka.gcd.org)
  */
public class KaffeEncoder extends Encoder
{

/*************************************************************************/

/*
 * Class Variables
 */
 
/*************************************************************************/

/*
 * Class Methods
 */

/*************************************************************************/

/*
 * Constructors
 */
public
KaffeEncoder(OutputStream out, String enc)
{
  super(out);
  try {
      converter = CharToByteConverter.getConverter(enc);
  }
  catch (java.io.UnsupportedEncodingException _) {
      converter = null;
  }
}

/*************************************************************************/

/*
 * Instance Variables
 */

CharToByteConverter converter;
 
/*************************************************************************/

/*
 * Instance Methods
 */

/**
  * This method is supposed to return the number of bytes that can be
  * converted out of the <code>len</code> chars in the specified array
  * starting at <code>offset</code>.  But it is impossible to calculate
  * such numbber without actually executing the conversion.
  * So, this method always throws UnsupportedException.
  */
public int
bytesInCharArray(char[] buf, int offset, int len)
{
  throw new UnsupportedOperationException();
}

/*************************************************************************/

/**
  * Convert the requested chars to bytes
  */
public byte[]
convertToBytes(char[] buf, int offset, int len) throws CharConversionException
{

  ByteArrayOutputStream tmpout = new ByteArrayOutputStream(len * 3);
  byte[] bbuf = new byte[len * 7];
  int bbuflen = converter.convert(buf, offset, len, bbuf, 0, bbuf.length);
  while (bbuflen > 0) {
      tmpout.write(bbuf, 0, bbuflen);
      bbuflen = converter.flush(bbuf, 0, bbuf.length);
  }
  return (tmpout.toByteArray());
}

/**
  * Convert the requested chars to bytes
  */
public byte[]
convertToBytes(char[] buf, int buf_offset, int len, byte[] bbuf, 
               int bbuf_offset) throws CharConversionException
{
  converter.convert(buf, buf_offset, len,
                    bbuf, bbuf_offset, bbuf.length - bbuf_offset);
  if (converter.flush(new byte[8], 0, 8) > 0) {
      throw new ArrayIndexOutOfBoundsException();
  }
  return(bbuf);
}

/*************************************************************************/

/**
  * Write the requested number of chars to the underlying stream
  */
public void
write(char[] buf, int offset, int len) throws IOException
{
  out.write(convertToBytes(buf, offset, len));
}

} // class KaffEncoder

